/* BdfServer - Copyright (c) 2025 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.impl;

import fr.exemole.bdfserver.api.managers.JsLibManager;
import fr.exemole.bdfserver.api.storage.BdfStorageException;
import fr.exemole.bdfserver.tools.jslib.JsLibConf;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.SortedMap;
import java.util.TreeMap;
import net.mapeadores.util.ini.IniParser;
import net.mapeadores.util.io.DocStream;
import net.mapeadores.util.io.ResourceStorages;
import net.mapeadores.util.jslib.JsLib;
import net.mapeadores.util.jslib.JsLibBuilder;
import net.mapeadores.util.jslib.JsLibUtils;
import net.mapeadores.util.jslib.TemplateFamily;
import net.mapeadores.util.jslib.TemplateFamilyBuilder;
import net.mapeadores.util.text.RelativePath;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
class JsLibManagerImpl implements JsLibManager {

    private final static RelativePath JSLIBS_DIR = RelativePath.build("jslibs");
    private final ResourceStorages resourceStorages;
    private final Map<String, List<JsLib>> libByPageMap = new HashMap<String, List<JsLib>>();

    JsLibManagerImpl(ResourceStorages resourceStorages) {
        this.resourceStorages = resourceStorages;
    }

    @Override
    public synchronized void update() {
        SortedMap<String, JsLibBuffer> bufferMap = new TreeMap<String, JsLibBuffer>();
        SortedMap<String, RelativePath> jsLibFolderMap = resourceStorages.listSubFolders(JSLIBS_DIR);
        for (Map.Entry<String, RelativePath> entry : jsLibFolderMap.entrySet()) {
            String name = entry.getKey();
            RelativePath dirPath = entry.getValue();
            JsLibConf jsLibConf = getConf(dirPath);
            if (jsLibConf != null) {
                JsLibBuffer jsLibBuffer = new JsLibBuffer(dirPath, name, jsLibConf);
                bufferMap.put(name, jsLibBuffer);
            }
        }
        for (JsLibBuffer jsLibBuffer : bufferMap.values()) {
            jsLibBuffer.build();
        }
        for (JsLibBuffer jsLibBuffer : bufferMap.values()) {
            JsLib jsLib = jsLibBuffer.getJsLib();
            populateByPage(jsLib, jsLibBuffer.getPageSet());
        }
    }

    @Override
    public List<JsLib> getCustomJsLibList(String page) {
        List<JsLib> list = libByPageMap.get(page);
        if (list != null) {
            return list;
        } else {
            return JsLibUtils.EMPTY_JSLIBLIST;
        }
    }

    private JsLibConf getConf(RelativePath libDir) {
        RelativePath iniPath = libDir.buildChild("jslib.ini");
        if (!resourceStorages.containsResource(iniPath)) {
            return null;
        }
        Map<String, String> iniMap = new HashMap<String, String>();
        parseIni(iniPath, iniMap);
        return new JsLibConf(iniMap);
    }

    private void parseIni(RelativePath relativePath, Map<String, String> iniMap) {
        DocStream docStream = resourceStorages.getResourceDocStream(relativePath);
        if (docStream != null) {
            try (InputStream is = docStream.getInputStream()) {
                IniParser.parseIni(is, iniMap);
            } catch (IOException ioe) {
                throw new BdfStorageException(ioe);
            }
        }
    }

    private void populateByPage(JsLib jsLib, Set<String> pageSet) {
        for (String page : pageSet) {
            List<JsLib> currentList = libByPageMap.get(page);
            if (currentList == null) {
                currentList = new ArrayList<JsLib>();
                libByPageMap.put(page, currentList);
            }
            currentList.add(jsLib);
        }
    }


    private class JsLibBuffer {

        private final RelativePath rootPath;
        private final String name;
        private final JsLibConf jsLibConf;
        private final JsLibBuilder jsLibBuilder = new JsLibBuilder();
        private final Set<String> pageSet = new HashSet<String>();

        private JsLibBuffer(RelativePath rootPath, String name, JsLibConf jsLibConf) {
            this.rootPath = rootPath;
            this.name = name;
            this.jsLibConf = jsLibConf;
        }

        private void build() {
            for (String page : getValues("use.txt")) {
                pageSet.add(page);
            }
            for (String jsPath : getValues("js.txt")) {
                try {
                    jsLibBuilder.addJsScript(convertToRelativePath(jsPath));
                } catch (ParseException pe) {

                }
            }
            for (String cssPath : getValues("css.txt")) {
                try {
                    jsLibBuilder.addCssPath(convertToRelativePath(cssPath).getPath());
                } catch (ParseException pe) {

                }
            }
            SortedMap<String, RelativePath> subFolders = resourceStorages.listSubFolders(rootPath);
            RelativePath templatesPath = subFolders.get("jslib-templates");
            if (templatesPath != null) {
                TemplateFamily jslibTemplateFamily = TemplateFamilyBuilder.init(name)
                        .setOriginObject(templatesPath)
                        .toTemplateFamily();
                jsLibBuilder.addTemplateFamily(jslibTemplateFamily);
            }
        }

        private Set<String> getPageSet() {
            return pageSet;
        }

        private JsLib getJsLib() {
            return jsLibBuilder.toJsLib();
        }

        private String[] getValues(String fileName) {
            RelativePath filePath = rootPath.buildChild(fileName);
            if (resourceStorages.containsResource(filePath)) {
                List<String> valueList = new ArrayList<String>();
                try (BufferedReader reader = new BufferedReader(new InputStreamReader(resourceStorages.getResourceDocStream(filePath).getInputStream(), "UTF-8"))) {
                    String ligne;
                    while ((ligne = reader.readLine()) != null) {
                        ligne = ligne.trim();
                        if (ligne.length() > 0) {
                            if (!ligne.startsWith("#")) {
                                valueList.add(ligne);
                            }
                        }
                    }

                } catch (IOException ioe) {
                }
                return valueList.toArray(new String[valueList.size()]);
            } else {
                return StringUtils.EMPTY_STRINGARRAY;
            }

        }

        private RelativePath convertToRelativePath(String path) throws ParseException {
            if (path.startsWith("/")) {
                return RelativePath.parse(path.substring(0));
            } else {
                return rootPath.parseChild(path);
            }
        }

    }

}
